@@ -1,4 +1,4 @@
-# $Id: Memcached.pm 601 2007-07-17 17:47:33Z bradfitz $
+# $Id: Memcached.pm 809 2009-05-03 03:23:23Z bradfitz $
 #
 # Copyright (c) 2003, 2004  Brad Fitzpatrick <brad@danga.com>
 #
@@ -35,7 +35,7 @@ use constant F_COMPRESS => 2;
 use constant COMPRESS_SAVINGS => 0.20; # percent
 
 use vars qw($VERSION $HAVE_ZLIB $FLAG_NOSIGNAL);
-$VERSION = "1.24";
+$VERSION = "1.25";
 
 BEGIN {
     $HAVE_ZLIB = eval "use Compress::Zlib (); 1;";
@@ -244,6 +244,7 @@ sub sock_to_host { # (host)
         # if a preferred IP is known, try that first.
         if ($self && $self->{pref_ip}{$ip}) {
             socket($sock, PF_INET, SOCK_STREAM, $proto);
+            $sock_map{$sock} = $host;
             my $prefip = $self->{pref_ip}{$ip};
             $sin = Socket::sockaddr_in($port,Socket::inet_aton($prefip));
             if (_connect_sock($sock,$sin,$self->{connect_timeout})) {
@@ -259,6 +260,7 @@ sub sock_to_host { # (host)
         # normal path, or fallback path if preferred IP failed
         unless ($connected) {
             socket($sock, PF_INET, SOCK_STREAM, $proto);
+            $sock_map{$sock} = $host;
             $sin = Socket::sockaddr_in($port,Socket::inet_aton($ip));
             my $timeout = $self ? $self->{connect_timeout} : 0.25;
             unless (_connect_sock($sock,$sin,$timeout)) {
@@ -269,6 +271,7 @@ sub sock_to_host { # (host)
         }
     } else { # it's a unix domain/local socket
         socket($sock, PF_UNIX, SOCK_STREAM, 0);
+        $sock_map{$sock} = $host;
         $sin = Socket::sockaddr_un($host);
         my $timeout = $self ? $self->{connect_timeout} : 0.25;
         unless (_connect_sock($sock,$sin,$timeout)) {
@@ -283,7 +286,6 @@ sub sock_to_host { # (host)
     $| = 1;
     select($old);
 
-    $sock_map{$sock} = $host;
     $cache_sock{$host} = $sock;
 
     return $sock;
@@ -328,6 +330,7 @@ sub disconnect_all {
         close $sock;
     }
     %cache_sock = ();
+    @buck2sock = ();
 }
 
 # writes a line, then reads result.  by default stops reading after a
@@ -421,7 +424,7 @@ sub delete {
         $self->{'stat_callback'}->($stime, $etime, $sock, 'delete');
     }
 
-    return $res eq "DELETED\r\n";
+    return defined $res && $res eq "DELETED\r\n";
 }
 *remove = \&delete;
 
@@ -457,6 +460,7 @@ sub _set {
         $val = Storable::nfreeze($val);
         $flags |= F_STORABLE;
     }
+    warn "value for memkey:$key is not defined" unless defined $val;
 
     my $len = length($val);
 
@@ -491,7 +495,7 @@ sub _set {
         $self->{'stat_callback'}->($stime, $etime, $sock, $cmdname);
     }
 
-    return $res eq "STORED\r\n";
+    return defined $res && $res eq "STORED\r\n";
 }
 
 sub incr {
@@ -522,7 +526,7 @@ sub _incrdecr {
         $self->{'stat_callback'}->($stime, $etime, $sock, $cmdname);
     }
 
-    return undef unless $res =~ /^(\d+)/;
+    return undef unless defined $res && $res =~ /^(\d+)/;
     return $1;
 }
 
@@ -783,7 +787,7 @@ sub flush_all {
     foreach my $host (@hosts) {
         my $sock = $self->sock_to_host($host);
         my @res = $self->run_command($sock, "flush_all\r\n");
-        $success = 0 unless (@res);
+        $success = 0 unless (scalar @res == 1 && (($res[0] || "") eq "OK\r\n"));
     }
 
     return $success;
@@ -894,7 +898,7 @@ sub stats_reset {
   HOST: foreach my $host (@{$self->{'buckets'}}) {
         my $sock = $self->sock_to_host($host);
         my $ok = _write_and_read($self, $sock, "stats reset");
-        unless ($ok eq "RESET\r\n") {
+        unless (defined $ok && $ok eq "RESET\r\n") {
             _dead_sock($sock);
         }
     }
